@@ -175,8 +175,6 @@ static gboolean totem_embedded_push_parser (gpointer data);
 static gboolean totem_embedded_play (TotemEmbedded *embedded, GError **error);
 static void totem_embedded_set_logo_by_name (TotemEmbedded *embedded, const char *name);
 
-static void totem_embedded_clear_playlist (TotemEmbedded *embedded);
-
 static void totem_embedded_update_menu (TotemEmbedded *emb);
 static void on_open1_activate (GtkButton *button, TotemEmbedded *emb);
 static void totem_embedded_toggle_fullscreen (TotemEmbedded *emb);
@@ -191,6 +189,7 @@ enum {
 	START_STREAM,
 	STOP_STREAM,
 	SIGNAL_TICK,
+	PROPERTY_CHANGE,
 	LAST_SIGNAL
 };
 
@@ -259,6 +258,14 @@ static void totem_embedded_class_init (TotemEmbeddedClass *klass)
 				NULL /* accu */, NULL /* accu data */,
 				totempluginviewer_marshal_VOID__UINT_UINT_STRING,
 				G_TYPE_NONE, 3, param_types);
+	signals[PROPERTY_CHANGE] =
+		g_signal_new ("property-change",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      0 /* class offset */,
+			      NULL /* accu */, NULL /* accu data */,
+			      totempluginviewer_marshal_VOID__STRING_BOXED,
+			      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_VALUE);
 }
 
 static void
@@ -281,10 +288,17 @@ static void
 totem_embedded_save_volume (TotemEmbedded *emb, double volume)
 {
 	GConfClient *gc;
+	GValue value = { 0, };
 
 	gc = gconf_client_get_default ();
 	gconf_client_set_int (gc, GCONF_PREFIX"/volume", (int) (volume * 100.0), NULL);
 	g_object_unref (G_OBJECT (gc));
+
+	g_value_init (&value, G_TYPE_DOUBLE);
+	g_value_set_double (&value, volume);
+	g_signal_emit (emb, signals[PROPERTY_CHANGE], 0,
+		       TOTEM_PROPERTY_VOLUME,
+		       &value);
 }
 
 static void
@@ -545,6 +559,11 @@ totem_embedded_stop (TotemEmbedded *emb,
 	totem_embedded_set_state (emb, TOTEM_STATE_STOPPED);
 	bacon_video_widget_stop (emb->bvw);
 
+	g_signal_emit (emb, signals[SIGNAL_TICK], 0,
+		       (guint32) 0,
+		       (guint32) 0,
+		       totem_states[emb->state]);
+
 	return TRUE;
 }
 
@@ -606,6 +625,7 @@ totem_embedded_set_volume (TotemEmbedded *embedded,
 {
 	g_message ("totem_embedded_set_volume: %f", volume);
 	bacon_video_widget_set_volume (embedded->bvw, volume);
+	totem_embedded_save_volume (embedded, volume);
 	return TRUE;
 }
 
@@ -943,15 +963,126 @@ totem_embedded_set_uri (TotemEmbedded *emb,
 	emb->stream_uri = NULL;
 }
 
+static void
+totem_pl_item_free (gpointer data, gpointer user_data)
+{
+	TotemPlItem *item = (TotemPlItem *) data;
+
+	if (!item)
+		return;
+	g_free (item->uri);
+	g_free (item);
+}
+
+static gboolean
+totem_embedded_clear_playlist (TotemEmbedded *embedded, GError *error)
+{
+	g_list_foreach (embedded->playlist, (GFunc) totem_pl_item_free, NULL);
+	g_list_free (embedded->playlist);
+
+	embedded->playlist = NULL;
+	embedded->current = NULL;
+	embedded->num_items = 0;
+
+	totem_embedded_set_uri (embedded, NULL, NULL, FALSE);
+
+	bacon_video_widget_close (embedded->bvw);
+
+	return TRUE;
+}
+
+static gboolean
+totem_embedded_add_item (TotemEmbedded *embedded, const char *uri, GError *error)
+{
+	TotemPlItem *item;
+
+	g_message ("totem_embedded_add_item: %s", uri);
+
+	item = g_new0 (TotemPlItem, 1);
+	item->uri = g_strdup (uri);
+	item->duration = -1;
+	item->starttime = -1;
+
+	embedded->playlist = g_list_append (embedded->playlist, item);
+	embedded->num_items++;
+
+	if (embedded->current_uri == NULL) {
+		embedded->current = embedded->playlist;
+		totem_embedded_set_uri (embedded,
+					(const char *) uri,
+					embedded->base_uri /* FIXME? */,
+					FALSE);
+		totem_embedded_open_internal (embedded, FALSE, NULL /* FIXME */);
+	}
+
+	return TRUE;
+}
+
+static gboolean
+totem_embedded_set_fullscreen (TotemEmbedded *emb,
+			       gboolean fullscreen_enabled,
+			       GError **error)
+{
+	GValue value = { 0, };
+	GtkAction *fs_action;
+	
+	fs_action = GTK_ACTION (gtk_builder_get_object 
+				(emb->menuxml, "fullscreen1"));
+
+	if (totem_fullscreen_is_fullscreen (emb->fs) == fullscreen_enabled)
+		return TRUE;
+
+	g_message ("totem_embedded_set_fullscreen: %d", fullscreen_enabled);
+
+	if (fullscreen_enabled == FALSE) {
+		GtkWidget * container;
+		container = GTK_WIDGET (gtk_builder_get_object (emb->xml,
+								"video_box"));
+
+		totem_fullscreen_set_fullscreen (emb->fs, FALSE);
+		gtk_widget_reparent (GTK_WIDGET (emb->bvw), container);
+		gtk_widget_hide_all (emb->fs_window);
+
+		gtk_action_set_sensitive (fs_action, TRUE);
+	} else {
+		GdkRectangle rect;
+		int monitor;
+
+		/* Move the fullscreen window to the screen where the
+		 * video widget currently is */
+		monitor = gdk_screen_get_monitor_at_window (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
+							    GTK_WIDGET (emb->bvw)->window);
+		gdk_screen_get_monitor_geometry (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
+						 monitor, &rect);
+		gtk_window_move (GTK_WINDOW (emb->fs_window), rect.x, rect.y);
+
+		gtk_widget_reparent (GTK_WIDGET (emb->bvw), emb->fs_window);
+		bacon_video_widget_set_fullscreen (emb->bvw, TRUE);
+		gtk_window_fullscreen (GTK_WINDOW (emb->fs_window));
+		totem_fullscreen_set_fullscreen (emb->fs, TRUE);
+		gtk_widget_show_all (emb->fs_window);
+
+		gtk_action_set_sensitive (fs_action, FALSE);
+	}
+
+	g_value_init (&value, G_TYPE_BOOLEAN);
+	g_value_set_boolean (&value, fullscreen_enabled);
+	g_signal_emit (emb, signals[PROPERTY_CHANGE], 0,
+		       TOTEM_PROPERTY_ISFULLSCREEN,
+		       &value);
+
+	return TRUE;
+}
+
 static gboolean
 totem_embedded_open_uri (TotemEmbedded *emb,
 			 const char *uri,
 			 const char *base_uri,
 			 GError **error)
 {
-	totem_embedded_clear_playlist (emb);
+	g_message ("totem_embedded_open_uri: uri %s base_uri: %s", uri, base_uri);
 
-	bacon_video_widget_close (emb->bvw);
+	totem_embedded_clear_playlist (emb, NULL);
 
 	totem_embedded_set_uri (emb, uri, base_uri, FALSE);
 
@@ -964,9 +1095,9 @@ totem_embedded_open_stream (TotemEmbedded *emb,
 			    const char *base_uri,
 			    GError **error)
 {
-	totem_embedded_clear_playlist (emb);
+	g_message ("totem_embedded_open_stream called: uri %s, base_uri: %s", uri, base_uri);
 
-	bacon_video_widget_close (emb->bvw);
+	totem_embedded_clear_playlist (emb, NULL);
 
 	totem_embedded_set_uri (emb, uri, base_uri, TRUE);
 	/* We can only have one item in the "playlist" when
@@ -1050,7 +1181,7 @@ totem_embedded_set_local_file (TotemEmbedded *emb,
 
 	g_message ("Setting the current path to %s", path);
 
-	totem_embedded_clear_playlist (emb);
+	totem_embedded_clear_playlist (emb, NULL);
 
 	file_uri = g_filename_to_uri (path, NULL, error);
 	if (!file_uri)
@@ -1096,7 +1227,7 @@ totem_embedded_set_playlist (TotemEmbedded *emb,
 	g_message ("Setting the current playlist to %s (base: %s)",
 		   path, base_uri);
 
-	totem_embedded_clear_playlist (emb);
+	totem_embedded_clear_playlist (emb, NULL);
 
 	file_uri = g_filename_to_uri (path, NULL, error);
 	if (!file_uri)
@@ -1129,21 +1260,27 @@ totem_embedded_update_menu (TotemEmbedded *emb)
 		emb->app = NULL;
 	}
 
-	if (emb->mimetype) {
+	if (emb->mimetype && strcmp (emb->mimetype, "application/octet-stream") != 0) {
 		emb->app = gnome_vfs_mime_get_default_application_for_uri
 				(emb->current_uri, emb->mimetype);
 	} else {
+		const char *uri;
+
+		/* If we're reading a local file, that's not a playlist,
+		 * we need to use that to get its proper mime-type */
+		if (emb->stream_uri != NULL)
+			uri = emb->stream_uri;
+		else
+			uri = emb->current_uri;
 		emb->app = gnome_vfs_mime_get_default_application_for_uri
-				(emb->current_uri,
-				 gnome_vfs_get_mime_type_for_name (emb->current_uri));
+				(uri, gnome_vfs_get_mime_type_for_name (uri));
 	}
 
 	if (emb->app == NULL) {
-
 		if (emb->mimetype != NULL) {
-			g_warning ("Mimetype '%s' doesn't have a handler", emb->mimetype);
+			g_message ("Mimetype '%s' doesn't have a handler", emb->mimetype);
 		} else {
-			g_warning ("No handler for URI '%s' (guessed mime-type '%s')",
+			g_message ("No handler for URI '%s' (guessed mime-type '%s')",
 				   emb->current_uri,
 				   gnome_vfs_get_mime_type_for_name (emb->current_uri));
 		}
@@ -1401,40 +1538,10 @@ on_got_redirect (GtkWidget *bvw, const char *mrl, TotemEmbedded *emb)
 static void
 totem_embedded_toggle_fullscreen (TotemEmbedded *emb)
 {
-	GtkAction * fs_action = GTK_ACTION (gtk_builder_get_object 
-					    (emb->menuxml, "fullscreen1"));
-
 	if (totem_fullscreen_is_fullscreen (emb->fs) != FALSE)
-	{
-		GtkWidget * container;
-		container = GTK_WIDGET (gtk_builder_get_object (emb->xml,
-								"video_box"));
-
-		totem_fullscreen_set_fullscreen (emb->fs, FALSE);
-		gtk_widget_reparent (GTK_WIDGET (emb->bvw), container);
-		gtk_widget_hide_all (emb->fs_window);
-		
-		gtk_action_set_sensitive (fs_action, TRUE);
-	} else {
-		GdkRectangle rect;
-		int monitor;
-
-		/* Move the fullscreen window to the screen where the
-		 * video widget currently is */
-		monitor = gdk_screen_get_monitor_at_window (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
-							    GTK_WIDGET (emb->bvw)->window);
-		gdk_screen_get_monitor_geometry (gtk_widget_get_screen (GTK_WIDGET (emb->bvw)),
-						 monitor, &rect);
-		gtk_window_move (GTK_WINDOW (emb->fs_window), rect.x, rect.y);
-
-		gtk_widget_reparent (GTK_WIDGET (emb->bvw), emb->fs_window);
-		bacon_video_widget_set_fullscreen (emb->bvw, TRUE);
-		gtk_window_fullscreen (GTK_WINDOW (emb->fs_window));
-		totem_fullscreen_set_fullscreen (emb->fs, TRUE);
-		gtk_widget_show_all (emb->fs_window);
-
-		gtk_action_set_sensitive (fs_action, FALSE);
-	}
+		totem_embedded_set_fullscreen (emb, FALSE, NULL);
+	else
+		totem_embedded_set_fullscreen (emb, TRUE, NULL);
 }
 
 static void
@@ -1579,6 +1686,7 @@ static void
 cb_vol (GtkWidget *val, gdouble value, TotemEmbedded *emb)
 {
 	bacon_video_widget_set_volume (emb->bvw, value);
+	totem_embedded_save_volume (emb, value);
 }
 
 static void
@@ -1610,7 +1718,9 @@ on_tick (GtkWidget *bvw,
 	}
 
 	g_signal_emit (emb, signals[SIGNAL_TICK], 0,
-		       (guint32) current_time, (guint32) stream_length, totem_states[emb->state]);
+		       (guint32) current_time,
+		       (guint32) stream_length,
+		       totem_states[emb->state]);
 }
 
 static void
@@ -1677,6 +1787,7 @@ totem_embedded_action_volume_relative (TotemEmbedded *emb, double off_pct)
 	
 	vol = bacon_video_widget_get_volume (emb->bvw);
 	bacon_video_widget_set_volume (emb->bvw, vol + off_pct);
+	totem_embedded_save_volume (emb, vol + off_pct);
 }
 
 static gboolean
@@ -1889,6 +2000,7 @@ totem_embedded_construct (TotemEmbedded *emb,
 	g_signal_connect (G_OBJECT (emb->volume), "value-changed",
 			  G_CALLBACK (cb_vol), emb);
 	bacon_video_widget_set_volume (emb->bvw, volume);
+	totem_embedded_save_volume (emb, volume);
 
 	emb->statusbar = TOTEM_STATUSBAR (gtk_builder_get_object (emb->xml, "statusbar"));
 	gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (emb->statusbar), FALSE);
@@ -2047,28 +2159,6 @@ totem_embedded_unset_window (TotemEmbedded *embedded,
 	return TRUE;
 }
 
-static void
-totem_pl_item_free (gpointer data, gpointer user_data)
-{
-	TotemPlItem *item = (TotemPlItem *) data;
-
-	if (!item)
-		return;
-	g_free (item->uri);
-	g_free (item);
-}
-
-static void
-totem_embedded_clear_playlist (TotemEmbedded *embedded)
-{
-	g_list_foreach (embedded->playlist, (GFunc) totem_pl_item_free, NULL);
-	g_list_free (embedded->playlist);
-
-	embedded->playlist = NULL;
-	embedded->current = NULL;
-	embedded->num_items = 0;
-}
-
 static void
 entry_metadata_foreach (const char *key,
 			const char *value,
@@ -2119,7 +2209,6 @@ totem_embedded_push_parser (gpointer data)
 	TotemPlParserResult res;
 
 	emb->parser_id = 0;
-	totem_embedded_clear_playlist (emb);
 
 	parser = totem_pl_parser_new ();
 	g_object_set (parser, "force", TRUE,
