@@ -24,7 +24,7 @@
  * permission are above and beyond the permissions granted by the GPL license
  * Totem is covered by.
  *
- * Monday 7th February 2005: Christian Schaller: Add excemption clause.
+ * Monday 7th February 2005: Christian Schaller: Add exception clause.
  * See license_change file for details.
  *
  */
@@ -35,8 +35,10 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
+#include <gconf/gconf-client.h>
 
 #include "totem-interface.h"
+#include "totem-private.h"
 
 static GtkWidget *
 totem_interface_error_dialog (const char *title, const char *reason,
@@ -96,43 +98,113 @@ totem_interface_error_blocking (const char *title, const char *reason,
 	gtk_widget_destroy (error_dialog);
 }
 
-GladeXML *
-totem_interface_load_with_root (const char *name, const char *root_widget,
-		const char *display_name, gboolean fatal, GtkWindow *parent)
+static void
+link_button_clicked_cb (GtkWidget *widget, Totem *totem)
 {
-	GladeXML *glade;
+	const char *uri;
+	char *command, *browser, *escaped_uri;
+	GError *error = NULL;
+
+	uri = gtk_link_button_get_uri (GTK_LINK_BUTTON (widget));
+	escaped_uri = g_shell_quote (uri);
+	browser = gconf_client_get_string (totem->gc, "/desktop/gnome/url-handlers/http/command", NULL);
+
+	if (browser == NULL || browser[0] == '\0') {
+		char *message;
+
+		message = g_strdup_printf(_("Could not launch URL \"%s\": %s"), uri, _("Default browser not configured"));
+		totem_interface_error (_("Error launching URI"), message, GTK_WINDOW (totem->win));
+		g_free (message);
+	} else {
+		char *message;
+
+		command = g_strdup_printf (browser, escaped_uri);
+		if (g_spawn_command_line_async ((const char*) command, &error) == FALSE) {
+			message = g_strdup_printf(_("Could not launch URL \"%s\": %s"), uri, error->message);
+			totem_interface_error (_("Error launching URI"), message, GTK_WINDOW (totem->win));
+			g_free (message);
+			g_error_free (error);
+		}
+		g_free (command);
+	}
+
+	g_free (escaped_uri);
+	g_free (browser);
+}
+
+void
+totem_interface_error_with_link (const char *title, const char *reason,
+				 const char *uri, const char *label, GtkWindow *parent, Totem *totem)
+{
+	GtkWidget *error_dialog, *link_button, *hbox;
+
+	if (label == NULL)
+		label = uri;
+
+	error_dialog = totem_interface_error_dialog (title, reason, parent);
+	link_button = gtk_link_button_new_with_label (uri, label);
+	g_signal_connect (G_OBJECT (link_button), "clicked", G_CALLBACK (link_button_clicked_cb), totem);
+
+	hbox = gtk_hbox_new (TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), link_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (error_dialog)->vbox), hbox, TRUE, FALSE, 0); 
+
+	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog), GTK_RESPONSE_OK);
+
+	g_signal_connect (G_OBJECT (error_dialog), "destroy", G_CALLBACK
+			(gtk_widget_destroy), error_dialog);
+	g_signal_connect (G_OBJECT (error_dialog), "response", G_CALLBACK
+			(gtk_widget_destroy), error_dialog);
+	gtk_window_set_modal (GTK_WINDOW (error_dialog), TRUE);
+
+	gtk_widget_show_all (error_dialog);
+}
+
+GtkBuilder *
+totem_interface_load (const char *name, gboolean fatal, GtkWindow *parent, gpointer user_data)
+{
+	GtkBuilder *builder = NULL;
 	char *filename;
 
-	glade = NULL;
 	filename = totem_interface_get_full_path (name);
-
-	if (filename != NULL)
-		glade = glade_xml_new (filename, root_widget, GETTEXT_PACKAGE);
+	builder = totem_interface_load_with_full_path (filename, fatal, parent,
+						       user_data);
 	g_free (filename);
 
-	if (glade == NULL)
+	return builder;
+}
+
+GtkBuilder *
+totem_interface_load_with_full_path (const char *filename, gboolean fatal, 
+				     GtkWindow *parent, gpointer user_data)
+{
+	GtkBuilder *builder = NULL;
+	GError *error = NULL;
+
+	if (filename != NULL) {
+		builder = gtk_builder_new ();
+		gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
+	}
+
+	if (builder == NULL || gtk_builder_add_from_file (builder, filename, &error) == FALSE)
 	{
 		char *msg;
 
-		msg = g_strdup_printf (_("Couldn't load the '%s' interface."), display_name);
+		msg = g_strdup_printf (_("Couldn't load the '%s' interface. %s"), filename, error->message);
 		if (fatal == FALSE)
 			totem_interface_error (msg, _("Make sure that Totem is properly installed."), parent);
 		else
 			totem_interface_error_blocking (msg, _("Make sure that Totem is properly installed."), parent);
 
 		g_free (msg);
+		g_error_free (error);
+
 		return NULL;
 	}
 
-	return glade;
-}
+	gtk_builder_connect_signals (builder, user_data);
 
-GladeXML *
-totem_interface_load (const char *name, const char *display_name,
-		gboolean fatal, GtkWindow *parent)
-{
-	return totem_interface_load_with_root (name, NULL, display_name,
-			fatal, parent);
+	return builder;
 }
 
 GdkPixbuf*
@@ -155,7 +227,7 @@ totem_interface_get_full_path (const char *name)
 	char *filename;
 
 #ifdef TOTEM_RUN_IN_SOURCE_TREE
-	/* Try the glade file in the source tree first */
+	/* Try the GtkBuilder file in the source tree first */
 	filename = g_build_filename ("..", "data", name, NULL);
 	if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE)
 	{
@@ -196,7 +268,7 @@ totem_gtk_plug_get_toplevel (GtkPlug *plug)
 					&parent, &children, &nchildren) == 0)
 		{
 			g_warning ("Couldn't find window manager window");
-			return None;
+			return NULL;
 		}
 
 		if (root == parent) {
@@ -232,7 +304,7 @@ totem_interface_set_transient_for (GtkWindow *window, GtkWindow *parent)
 char *
 totem_interface_get_license (void)
 {
-	const char const *license[] = {
+	const char *license[] = {
 		N_("Totem is free software; you can redistribute it and/or modify "
 		   "it under the terms of the GNU General Public License as published by "
 		   "the Free Software Foundation; either version 2 of the License, or "
@@ -255,3 +327,71 @@ totem_interface_get_license (void)
 			  NULL);
 }
 
+void
+totem_interface_boldify_label (GtkBuilder *builder, const char *name)
+{
+	GtkLabel *widget;
+	gchar *str_final;
+
+	widget = GTK_LABEL (gtk_builder_get_object (builder, name));
+
+	if (widget == NULL) {
+		g_warning ("widget '%s' not found", name);
+		return;
+	}
+
+	/* this way is probably better, but for some reason doesn't work with
+	 * labels with mnemonics.
+
+	static PangoAttrList *pattrlist = NULL;
+
+	if (pattrlist == NULL) {
+		PangoAttribute *attr;
+
+		pattrlist = pango_attr_list_new ();
+		attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+		attr->start_index = 0;
+		attr->end_index = G_MAXINT;
+		pango_attr_list_insert (pattrlist, attr);
+	}
+	gtk_label_set_attributes (widget, pattrlist);*/
+
+	str_final = g_strdup_printf ("<b>%s</b>", gtk_label_get_label (widget));
+	gtk_label_set_markup_with_mnemonic (widget, str_final);
+	g_free (str_final);
+}
+
+void
+totem_interface_italicise_label (GtkBuilder *builder, const char *name)
+{
+	GtkLabel *widget;
+	gchar *str_final;
+
+	widget = GTK_LABEL (gtk_builder_get_object (builder, name));
+
+	if (widget == NULL) {
+		g_warning ("widget '%s' not found", name);
+		return;
+	}
+
+	/* this way is probably better, but for some reason doesn't work with
+	 * labels with mnemonics.
+
+	static PangoAttrList *pattrlist = NULL;
+
+	if (pattrlist == NULL) {
+		PangoAttribute *attr;
+
+		pattrlist = pango_attr_list_new ();
+		attr = pango_attr_style_new (PANGO_STYLE_ITALIC);
+		attr->start_index = 0;
+		attr->end_index = G_MAXINT;
+		pango_attr_list_insert (pattrlist, attr);
+	}
+	gtk_label_set_attributes (widget, pattrlist);*/
+
+	str_final = g_strdup_printf ("<i>%s</i>", gtk_label_get_label (widget));
+	gtk_label_set_markup_with_mnemonic (widget, str_final);
+	g_free (str_final);
+}
+
